@@ -28,12 +28,12 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <string.h>
 
-#include <gnome.h>
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkx.h>
-#include <libgnomeui/gnome-password-dialog.h>
+#include "copy-paste/gnome-password-dialog.h"
 #include <gnome-keyring.h>
 #include <libnotify/notify.h>
 
@@ -51,9 +51,14 @@
 #include <libhal.h>
 #include <libhal-storage.h>
 
-#ifndef sun
+#if !defined(sun) && !defined(__FreeBSD__)
 #include <mntent.h>
-#else
+#elif defined(__FreeBSD__)
+#include <fstab.h>
+#include <sys/param.h>
+#include <sys/ucred.h>
+#include <sys/mount.h>
+#elif defined(sun)
 #include <sys/mnttab.h>
 #endif
 
@@ -582,19 +587,24 @@ static char *
 get_mntent_mount_point(const char *device_file)
 {
 	char *mount_point;
+#if! defined(sun) && !defined(__FreeBSD__)
 	FILE *f;
-#ifndef sun
 	struct mntent mnt;
 	struct mntent *mnte;
 	char buf[512];
-#else
+#elif defined(__FreeBSD__)
+	struct statfs *mounts;
+	int n_mounts;
+	int i;
+#elif defined(sun)
+	FILE *f;
 	struct mnttab mnt;
 	struct mnttab mpref = { NULL, NULL, NULL, NULL, NULL };
 #endif
 
 	mount_point = NULL;
 
-#ifndef sun
+#if !defined(sun) && !defined(__FreeBSD__)
 	if ((f = setmntent ("/proc/mounts", "r")) != NULL) {
 		
 		while ((mnte = getmntent_r (f, &mnt, buf, sizeof(buf))) != NULL) {
@@ -608,7 +618,7 @@ get_mntent_mount_point(const char *device_file)
 		endmntent (f);
 	}
 
-#else /* sun */
+#elif defined(sun) /* sun */
 
 	if ((f = fopen(MNTTAB, "r")) != NULL) {
 		mpref.mnt_special = (char *)device_file;
@@ -617,8 +627,13 @@ get_mntent_mount_point(const char *device_file)
 		}
 		fclose(f);
 	}
-
-#endif /* sun */
+#elif defined(__FreeBSD__)
+	n_mounts = getmntinfo(&mounts, MNT_NOWAIT);
+	for (i = 0; i < n_mounts; i++) {
+		if (!strcmp(mounts[i].f_mntfromname, device_file))
+			mount_point = g_strdup (mounts[i].f_mntonname);
+	}
+#endif /* sun && __FreeBSD__ */
 
 out:
 	return (mount_point);
@@ -1318,11 +1333,25 @@ volume_mount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 		fstype_override = g_strdup (fstype_from_command_line);
 	}
 
+	/* let a file system specify the override unless set already; e.g. for ntfs we want to override to ntfs-3g */
+	if (fstype_override == NULL) {
+		key = g_strdup_printf ("/system/storage/default_options/%s/fstype_override", fstype);
+		fstype_override = gconf_client_get_string (gconf_client, key, NULL);
+		g_free (key);
+		if (fstype_override != NULL && strlen (fstype_override) == 0) {
+			g_free (fstype_override);
+			fstype_override = NULL;
+		}
+	}
 
 	if (volume == NULL && (mount_options->len == 0)) {
 		/* volume from a non-pollable drive, just set uid.. */
 		
+#ifndef __FreeBSD__
 		snprintf (uidbuf, sizeof (uidbuf) - 1, "uid=%u", getuid ());
+#else
+		snprintf (uidbuf, sizeof (uidbuf) - 1, "-u=%u", getuid ());
+#endif
 		g_ptr_array_add (mount_options, uidbuf);
 		
 	} else if (((fstype_override != NULL) || (fstype != NULL)) && (mount_options->len == 0)) {
@@ -1349,10 +1378,22 @@ volume_mount (const char *udi, LibHalVolume *volume, LibHalDrive *drive)
 				g_debug ("read default option '%s' from gconf strlist key %s", option, key);
 
 				/* special workaround to replace "uid=" with "uid=<actual uid of caller>" */
+#ifndef __FreeBSD__
 				if (strcmp (option, "uid=") == 0) {
 					g_free (option);
 					option = g_strdup_printf ("uid=%u", getuid ());
 				}
+#else
+				if (strcmp (option, "-u=") == 0) {
+					g_free (option);
+					option = g_strdup_printf ("-u=%u", getuid ());
+				}
+#endif
+				/* use the same kind of workaround for the "locale=" option (used by ntfs-3g) */
+				if (strcmp (option, "locale=") == 0) {
+					g_free (option);
+					option = g_strdup_printf ("locale=%s", setlocale (LC_ALL, ""));
+				}
 				g_ptr_array_add (mount_options, option);
 
 				g_slist_free_1 (l);
@@ -2664,9 +2705,7 @@ main (int argc, char *argv[])
 	}
 
 	if (!opt_noui) {
-		gnome_program_init ("gnome-mount", VERSION, LIBGNOMEUI_MODULE,
-				    argc, argv, 
-				    GNOME_PARAM_NONE);
+		gtk_init (&argc, &argv);
 #ifdef ENABLE_NOTIFY
 		notify_init ("gnome-mount");
 #endif
